# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group


class World(object):
    """
    """
    def __init__(self, ):
        """
        """
        pass
